#!/usr/bin/env ruby
# Time-stamp: "2001-03-31 01:58:47 susho"

=begin
    Copyright (C) 2001  Masahiro Sakai

        Masahiro Sakai                  <zvm01052@nifty.ne.jp>

    You can redistribute this file and/or modify it under the same term
    of Ruby.  License of Ruby is included with Ruby distribution in
    the file "README".
=end


class MOFile
  class InvalidFormat < RuntimeError; end;

  Header = Struct.new(:magic,
		      :revision,
		      :nstrings,
		      :orig_table_offset,
		      :translated_table_offset,
		      :hash_table_size,
		      :hash_table_offset)

  def MOFile.open(*args, &block)
    self.new(*args, &block)
  end


  def initialize(file)
    if file.is_a? String
      file = File.new(file)
    else
      file = file
    end

    magic = file.read(4)
    case magic
    when "\x95\x04\x12\xde"
      @little_endian = false
    when "\xde\x12\x04\x95"
      @little_endian = true
    else
      raise InvalidFormat, "Invalid magic"
    end

    @header =
      Header.new(magic,
		 *(file.read(4 * 6).unpack(@little_endian ? 'V6' : 'N6')))
    
    file.pos = @header.orig_table_offset
    orig_table_data = file.read(8 * @header.nstrings).unpack(@little_endian ? 'V*' : 'N*')

    file.pos = @header.translated_table_offset
    trans_table_data = file.read(8 * @header.nstrings).unpack(@little_endian ? 'V*' : 'N*')

    original_strings = []
    for i in 0..(@header.nstrings - 1)
      file.pos = orig_table_data[i * 2 + 1]
      original_strings << file.read(orig_table_data[i * 2 + 0])
    end

    translated_strings = []
    for i in 0..(@header.nstrings - 1)
      file.pos = trans_table_data[i * 2 + 1]
      translated_strings << file.read(trans_table_data[i * 2 + 0])
    end

    @hash = Hash.new()
    for i in 0..(@header.nstrings - 1)
      @hash[original_strings[i]] = translated_strings[i]
    end
  end

  
  def [](key)
    @hash[key]
  end

  def to_hash
    @hash.dup
  end
end


# Test

if $0 == __FILE__
  if (ARGV.include? "-h") or (ARGV.include? "--help")
    STDERR.puts("mo.rb [filename.mo ...]")
    exit
  end

  ARGV.each{ |item|
    MOFile.open(item).to_hash.each{
      |key, value|
      puts("original message = \"#{key}\"")
      puts("translated message = \"#{value}\"")
      puts("--------------------------------------------------------------------")
    }
  }
end
